@@ -37,7 +37,7 @@
        Keith Owens <kaos@ocs.com.au> April 2000.
    */
 
-#ident "$Id: depmod.c 1.15 Sun, 09 Jul 2000 14:19:11 +1000 kaos $"
+#ident "$Id: depmod.c 1.18 Mon, 07 Aug 2000 18:09:03 +1000 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -75,6 +75,14 @@ typedef struct SYMBOL{
 	char status;
 } SYMBOL;
 
+/* Extracted from 2.4.0-test6-pre6/include/linux/pci.h */
+struct pci_device_id {
+	unsigned int vendor, device;		/* Vendor and device ID or PCI_ANY_ID */
+	unsigned int subvendor, subdevice;	/* Subsystem ID's or PCI_ANY_ID */
+	unsigned int class, class_mask;		/* (class,subclass,prog-if) triplet */
+	unsigned long driver_data;		/* Data private to the driver */
+};
+
 typedef struct MODULE {
 	char *name;
 	int resolved;
@@ -82,6 +90,8 @@ typedef struct MODULE {
 		SYMBOL **symtab;
 		int n_syms;
 	} defsym, undefs;
+	struct pci_device_id *pci;
+	int n_pci;
 } MODULE;
 
 typedef enum SYM_STATUS {
@@ -106,7 +116,7 @@ static LIST_SYMBOL *chunk;
 
 static int quiet;	/* Don't print errors */
 static int showerror;	/* Shows undefined symbols */
-static int verbose;	/* Print all modules visiteds */
+int flag_verbose = 0;
 
 extern int depmod_main(int argc, char **argv);
 extern int depmod_main_32(int argc, char **argv);
@@ -252,6 +262,38 @@ static void resolve(void)
 	}
 }
 
+/*
+ *	Verify that an "address" falls within the module image.
+ *	Note: assumes same machine and arch for depmod and module.
+ */
+static int in_range(const struct obj_file *f, unsigned long m_size,
+		    ElfW(Addr) start, unsigned int length)
+{
+	return(start >= f->baseaddr && (start + length - 1 - f->baseaddr) <= m_size);
+}
+
+/*
+ *	Extract any pci_device_table from the module.
+ *	Note: assumes same machine and arch for depmod and module.
+ */
+static void extract_pci_device_id(struct obj_file *f, void *image, unsigned long m_size, MODULE *mod)
+{
+	struct pci_device_id pci;
+	ElfW(Addr) ref_pci;
+	ElfW(Addr) ref_ref_pci = obj_symbol_final_value(f, obj_find_symbol(f, "__module_pci_device_table"));
+	if (!in_range(f, m_size, ref_ref_pci, sizeof(ref_pci)))
+		return;
+	memcpy(&ref_pci, (char *)image + ref_ref_pci - f->baseaddr, sizeof(ref_pci));
+	while (in_range(f, m_size, ref_pci, sizeof(pci))) {
+		memcpy(&pci, (char *)image + ref_pci - f->baseaddr, sizeof(pci));
+		ref_pci += sizeof(pci);
+		if (!pci.vendor)
+			break;
+		mod->pci = xrealloc(mod->pci, ++(mod->n_pci)*sizeof(*(mod->pci)));
+		mod->pci[mod->n_pci-1] = pci;
+	}
+}
+
 /*
  *	Read the symbols in an object and register them in the symbol table.
  *	Return -1 if there is an error.
@@ -274,6 +316,8 @@ static int loadobj(const char *objname, int ignore_suffix)
 	int n_undefs = 0;
 	int n_defsym = 0;
 	char *p;
+	void *image;
+	unsigned long m_size;
 
 	p = strrchr(objname, '/');
 	len = 1 + (int)(p - objname);
@@ -406,8 +450,25 @@ static int loadobj(const char *objname, int ignore_suffix)
 		mod->resolved = 1;
 	}
 
-	obj_free(f);
+	/* Do a pseudo relocation to base address 0x1000 (arbitrary).
+	 * All undefined symbols are treated as absolute 0.  This builds
+	 * enough of a module to allow extraction of internal data such
+	 * as device tables.
+	 */
+	obj_clear_undefineds(f);
+	obj_allocate_commons(f);
+	arch_create_got(f);
+	m_size = obj_load_size(f);
+	if (!obj_relocate(f, 0x1000)) {
+		error("depmod obj_relocate failed\n");
+		return(-1);
+	}
+	image = xmalloc(m_size);
+	obj_create_image(f, image);
+	extract_pci_device_id(f, image, m_size, mod);
+	free(image);
 
+	obj_free(f);
 	return 0;
 }
 
@@ -420,7 +481,6 @@ static int loadobj(const char *objname, int ignore_suffix)
  * OR
  * from a file containing kernel symbols from a different kernel.
  * The file can be either a "System.map" file or a copy of "/proc/ksyms".
- * Use a copy of "/proc/ksyms" if you are using versioned symbols!
  *
  * Return -1 if any error.
  */
@@ -528,25 +588,35 @@ static int addksyms(char *file_syms)
 /*
  *	Format the dependancy list of a module into a simple makefile.
  *	Print the dependancies in the depfile (or stdout if depfile is NULL).
+ *	Print the pcimap in the pcimapfile (or stdout if pcimapfile is NULL).
  */
-static void prtdepend(const char *pdepfile, char *base_dir)
+static void prtdepend(const char *pdepfile, const char *ppcimapfile, char *base_dir)
 {
-	FILE *out = stdout;
+	FILE *dep = stdout;
+	FILE *pcimap = stdout;
 	MODULE **tbdep;
-	MODULE *ptmod = modules;
+	MODULE *ptmod;
 	int i;
 	int skipchars;	/* For depmod -a in image of a tree */
 
 	if (pdepfile != NULL) {
-		if ((out = fopen(pdepfile, "w")) == NULL) {
+		if ((dep = fopen(pdepfile, "w")) == NULL) {
 			error("Can't open %s for writing", pdepfile);
 			exit(-1);
 		}
 	}
 
+	if (ppcimapfile != NULL) {
+		if ((pcimap = fopen(ppcimapfile, "w")) == NULL) {
+			error("Can't open %s for writing", ppcimapfile);
+			exit(-1);
+		}
+	}
+
 	skipchars = (base_dir ? strlen(base_dir) : 0);
 	tbdep = (MODULE **)alloca(sizeof(MODULE) * n_modules);
 
+	ptmod = modules;
 	for (i = 0; i < n_modules; i++, ptmod++) {
 		SYMBOL **undefs = ptmod->undefs.symtab;
 		int n_undefs = ptmod->undefs.n_syms;
@@ -585,18 +655,63 @@ static void prtdepend(const char *pdepfile, char *base_dir)
 				nberr++;
 			}
 		}
+		verbose("%s\n", ptmod->name + skipchars);
 
-		if (verbose)
-			printf("%s\n", ptmod->name + skipchars);
-
-		fprintf(out, "%s:", ptmod->name + skipchars);
+		fprintf(dep, "%s:", ptmod->name + skipchars);
 		for (m = 0; m < nbdepmod; m++) {
 			if (m != 0 /*&& (m & 3) == 0*/)
-				fprintf(out, " \\\n");
-			fprintf(out, "\t%s", tbdep[m]->name + skipchars);
+				fprintf(dep, " \\\n");
+			fprintf(dep, "\t%s", tbdep[m]->name + skipchars);
 		}
-		fprintf(out, "\n\n");
+		fprintf(dep, "\n\n");
 	}
+
+	ptmod = modules;
+	fprintf(pcimap, "# module             vendor     device     subvendor  subdevice  class      class_mask driver_data\n");
+	for (i = 0; i < n_modules; i++, ptmod++) {
+		int j, l;
+		struct pci_device_id *pci = ptmod->pci;
+		char *name, *shortname;
+		if (!ptmod->n_pci)
+			continue;
+		name = xstrdup(ptmod->name);
+		shortname = strrchr(name, '/');
+		if (!shortname)
+			shortname = name;	/* should never happen */
+		else {
+			++shortname;
+			l = strlen(shortname);
+			if (l > 3 && strcmp(shortname+l-3, ".gz") == 0) {
+				*(shortname+l-3) = '\0';
+				l -= 3;
+			}
+			if (l > 2 && strcmp(shortname+l-2, ".o") == 0) {
+				*(shortname+l-2) = '\0';
+				l -= 2;
+			} else if (l > 4 && strcmp(shortname+l-4, ".mod") == 0) {
+				*(shortname+l-4) = '\0';
+				l -= 4;
+			}
+		}
+		for (j = 0; j < ptmod->n_pci; j++, pci++) {
+			fprintf(pcimap, "%-20s 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*lx\n",
+				shortname,
+				2*sizeof(pci->vendor), pci->vendor,
+				2*sizeof(pci->device), pci->device,
+				2*sizeof(pci->subvendor), pci->subvendor,
+				2*sizeof(pci->subdevice), pci->subdevice,
+				2*sizeof(pci->class), pci->class,
+				2*sizeof(pci->class_mask), pci->class_mask,
+				2*sizeof(pci->driver_data), pci->driver_data);
+		}
+		free(name);
+	}
+
+	/* Close depfile last, critical timestamp */
+	if (ppcimapfile != NULL)
+		fclose(pcimap);
+	if (pdepfile != NULL)
+		fclose(dep);
 }
 
 /* For common 3264, only compile the usage message once, in the 64 bit version */
@@ -741,7 +856,7 @@ int DEPMOD_MAIN(int argc, char **argv)
 			break;
 
 		case 'v':
-			verbose = 1;
+			flag_verbose = 1;
 			break;
 
 		case 'V':
@@ -785,7 +900,7 @@ int DEPMOD_MAIN(int argc, char **argv)
 			for (i = 0; g && i < g->pathc; i++)
 				loadobj(g->pathv[i], ignore_suffix);
 			resolve();
-			prtdepend(nflag ? NULL : depfile, base_dir);
+			prtdepend(nflag ? NULL : depfile, nflag ? NULL : pcimapfile, base_dir);
 		}
 	} else {
 		/* not stdmode */
@@ -793,7 +908,7 @@ int DEPMOD_MAIN(int argc, char **argv)
 			for (; argc > 0 && ret != -1; ++argv, --argc)
 				loadobj(*argv, ignore_suffix);
 			resolve();
-			prtdepend(NULL, base_dir);
+			prtdepend(NULL, NULL, base_dir);
 		}
 	}
 
