@@ -25,6 +25,7 @@
 #ifndef _CONFIG_H
 #define _CONFIG_H
 
+#include <stdio.h>
 #include <sys/utsname.h>
 
 #define ETC_MODULES_CONF	"/etc/modules.conf"
@@ -63,11 +64,14 @@ extern struct EXEC_TYPE *execs;
 extern int nexecs;
 extern char *insmod_opt;
 extern char *depfile;
+extern char *pcimapfile;
 extern char *config_file;
 extern char *optlist[];
+extern char *prune[];
 extern OPT_LIST *opt_list;
 extern OPT_LIST *abovelist;
 extern OPT_LIST *belowlist;
+extern OPT_LIST *prunelist;
 extern OPT_LIST *probe_list;
 extern OPT_LIST *probeall_list;
 extern OPT_LIST *aliases;
