@@ -24,7 +24,7 @@
 #ifndef MODUTILS_OBJ_H
 #define MODUTILS_OBJ_H 1
 
-#ident "$Id: obj.h 1.8 Tue, 25 Jul 2000 16:31:08 +1000 kaos $"
+#ident "$Id: obj.h 1.9 Mon, 07 Aug 2000 18:07:12 +1000 kaos $"
 
 /* The relocatable object is manipulated using elfin types.  */
 
@@ -151,6 +151,7 @@ struct obj_symbol_patch_struct
 #define obj_string_patch		ObjW(string_patch)
 #define obj_symbol_patch		ObjW(symbol_patch)
 #define obj_check_undefineds		ObjW(check_undefineds)
+#define obj_clear_undefineds		ObjW(clear_undefineds)
 #define obj_allocate_commons		ObjW(allocate_commons)
 #define obj_load_size			ObjW(load_size)
 #define obj_relocate			ObjW(relocate)
@@ -210,6 +211,8 @@ int obj_symbol_patch (struct obj_file *f, int secidx, ElfW(Addr) offset,
 
 int obj_check_undefineds (struct obj_file *f, int quiet);
 
+void obj_clear_undefineds (struct obj_file *f);
+
 void obj_allocate_commons (struct obj_file *f);
 
 unsigned long obj_load_size (struct obj_file *f);
