@@ -48,7 +48,7 @@
        Keith Owens <kaos@ocs.com.au> April 2000.
    */
 
-#ident "$Id: insmod.c 1.21 Sun, 09 Jul 2000 22:30:18 +1000 kaos $"
+#ident "$Id: insmod.c 1.24 Mon, 07 Aug 2000 18:07:12 +1000 kaos $"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -78,7 +78,6 @@
 
 static int flag_force_load = 0;
 static int flag_silent_poll = 0;
-static int flag_verbose = 0;
 static int flag_export = 1;
 static int flag_load_map = 0;
 static int flag_ksymoops = 1;
@@ -612,8 +611,8 @@ static void add_ksymoops_symbols(struct obj_file *f, const char *filename,
 {
 	struct obj_section *sec;
 	struct obj_symbol *sym;
-	char *name;
-	char str[STRVERSIONLEN];
+	char *name, *absolute_filename;
+	char str[STRVERSIONLEN], real[PATH_MAX];
 	const char symprefix[] = "__insmod_";
 	int i, l, lm_name, lfilename, use_ksymtab, version;
 	struct stat statbuf;
@@ -625,8 +624,19 @@ static void add_ksymoops_symbols(struct obj_file *f, const char *filename,
 		".bss"
 	};
 
+	if (realpath(filename, real)) {
+		absolute_filename = xstrdup(real);
+	}
+	else {
+		int save_errno = errno;
+		error("cannot get realpath for %s", filename);
+		errno = save_errno;
+		perror("");
+		absolute_filename = xstrdup(filename);
+	}
+
 	lm_name = strlen(m_name);
-	lfilename = strlen(filename);
+	lfilename = strlen(absolute_filename);
 
 	/* add to ksymtab if it already exists or there is no ksymtab and other symbols
 	 * are not to be exported.  otherwise leave ksymtab alone for now, the
@@ -651,11 +661,11 @@ static void add_ksymoops_symbols(struct obj_file *f, const char *filename,
 		    8+					/* version in dec */
 		    1;					/* nul */
 		name = xmalloc(l);
-		if (stat(filename, &statbuf) != 0)
+		if (stat(absolute_filename, &statbuf) != 0)
 			statbuf.st_mtime = 0;
 		version = get_module_version(f, str);	/* -1 if not found */
 		snprintf(name, l, "%s%s_O%s_M%0*lX_V%d",
-			 symprefix, m_name, filename,
+			 symprefix, m_name, absolute_filename,
 			 2*sizeof(statbuf.st_mtime), statbuf.st_mtime,
 			 version);
 		sym = obj_add_symbol(f, name, -1,
@@ -664,6 +674,7 @@ static void add_ksymoops_symbols(struct obj_file *f, const char *filename,
 		if (use_ksymtab)
 			add_ksymtab(f, sym);
 	}
+	free(absolute_filename);
 
 	/* tag the desired sections if size is non-zero */
 
@@ -1422,7 +1433,13 @@ int INSMOD_MAIN(int argc, char **argv)
 	/* Let the module know about the kernel symbols.  */
 	add_kernel_symbols(f);
 
-	/* Allocate common symbols, symbol tables, and string tables.  */
+	/* Allocate common symbols, symbol tables, and string tables.
+	 *
+	 * The calls marked DEPMOD indicate the bits of code that depmod
+	 * uses to do a pseudo relocation, ignoring undefined symbols.
+	 * Any changes made to the relocation sequence here should be
+	 * checked against depmod.
+	 */
 #ifdef COMPAT_2_0
 	if (k_new_syscalls
 	    ? !create_this_module(f, m_name)
@@ -1433,15 +1450,15 @@ int INSMOD_MAIN(int argc, char **argv)
 		goto out;
 #endif
 
-	if (!obj_check_undefineds(f, quiet))
+	if (!obj_check_undefineds(f, quiet))	/* DEPMOD, obj_clear_undefineds */
 		goto out;
-	obj_allocate_commons(f);
+	obj_allocate_commons(f);	/* DEPMOD */
 
 	if (optind < argc) {
 		if (!process_module_arguments(f, argc - optind, argv + optind))
 			goto out;
 	}
-	arch_create_got(f);
+	arch_create_got(f);	/* DEPMOD */
 	hide_special_symbols(f);
 	if (flag_ksymoops)
 		add_ksymoops_symbols(f, filename, m_name);
@@ -1463,7 +1480,7 @@ int INSMOD_MAIN(int argc, char **argv)
 		goto out;
 	}
 	/* Module has now finished growing; find its size and install it.  */
-	m_size = obj_load_size(f);
+	m_size = obj_load_size(f);	/* DEPMOD */
 
 	if (noload) {
 		/* Don't bother actually touching the kernel.  */
@@ -1496,7 +1513,7 @@ int INSMOD_MAIN(int argc, char **argv)
 		}
 	}
 
-	if (!obj_relocate(f, m_addr)) {
+	if (!obj_relocate(f, m_addr)) {	/* DEPMOD */
 		if (!noload)
 			delete_module(m_name);
 		goto out;
