@@ -83,7 +83,6 @@ struct LINK {
 
 
 static char *conf_file = NULL;
-static char verbose_flag = 0;
 static int runit = 1;
 static int quiet = 0;
 static int debug = 0;
@@ -91,17 +90,6 @@ static int debug = 0;
 static NODE *in_kernel;
 static NODE *in_depfile;
 
-static void verbose(const char *ctl,...)
-{
-	if (verbose_flag) {
-		va_list list;
-		va_start(list, ctl);
-		vprintf(ctl, list);
-		va_end(list);
-		fflush(stdout);
-	}
-}
-
 /*
  *	Strip the path and extension off a file name
  */
@@ -623,6 +611,18 @@ static void config_show()
 		}
 	}
 
+	if (prunelist) {
+		puts("# Prune");
+		for (op = prunelist; op->name; ++op) {
+			GLOB_LIST *g = op->opts;
+			int n;
+			printf("prune %s", op->name);
+			for (n = 0; n < g->pathc; ++n)
+				printf(" %s", g->pathv[n]);
+			printf("\n");
+		}
+	}
+
 	if (abovelist) {
 		puts("# Above");
 		for (op = abovelist; op->name; ++op) {
@@ -1529,7 +1529,7 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'v':
-			verbose_flag = 1;
+			flag_verbose = 1;
 			break;
 
 		case 'k':
