@@ -18,7 +18,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_mips.c 1.2 Thu, 13 Apr 2000 18:17:59 +1000 kaos $"
+#ident "$Id: obj_mips.c 1.4 Sun, 09 Jul 2000 14:46:03 +1000 kaos $"
 
 #include <string.h>
 #include <stdlib.h>
@@ -70,7 +70,7 @@ arch_new_symbol (void)
 }
 
 int
-arch_load_proc_section(struct obj_section *sec, FILE *fp)
+arch_load_proc_section(struct obj_section *sec, int fp)
 {
   switch (sec->header.sh_type)
     {
@@ -219,3 +219,14 @@ arch_init_module (struct obj_file *f, struct module *mod)
 {
   return 1;
 }
+
+int
+arch_finalize_section_address(struct obj_file *f, Elf32_Addr base)
+{
+  int  i, n = f->header.e_shnum;
+
+  f->baseaddr = base;
+  for (i = 0; i < n; ++i)
+    f->sections[i]->header.sh_addr += base;
+  return 1;
+}
