@@ -19,7 +19,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_reloc.c 1.7 Tue, 25 Jul 2000 16:31:08 +1000 kaos $"
+#ident "$Id: obj_reloc.c 1.8 Mon, 07 Aug 2000 18:07:12 +1000 kaos $"
 
 #include <string.h>
 #include <assert.h>
@@ -107,6 +107,22 @@ obj_check_undefineds(struct obj_file *f, int quiet)
   return ret;
 }
 
+void
+obj_clear_undefineds(struct obj_file *f)
+{
+  unsigned long i;
+  struct obj_symbol *sym;
+  for (i = 0; i < HASH_BUCKETS; ++i)
+    {
+      for (sym = f->symtab[i]; sym ; sym = sym->next)
+	if (sym->secidx == SHN_UNDEF)
+	  {
+	    sym->secidx = SHN_ABS;
+	    sym->value = 0;
+	  }
+    }
+}
+
 void
 obj_allocate_commons(struct obj_file *f)
 {
