@@ -54,10 +54,10 @@
 #include <string.h>
 #include <unistd.h>
 #include <ctype.h>
-#include <ftw.h>
 #include <sys/stat.h>
 #include <signal.h>
 #include <limits.h>
+#include <sys/param.h>
 #include <errno.h>
 
 #include "util.h"
@@ -85,6 +85,9 @@ static int n_abovelist;
 OPT_LIST *belowlist;
 static int n_belowlist;
 
+OPT_LIST *prunelist;
+static int n_prunelist;
+
 OPT_LIST *probe_list;
 static int n_probe_list;
 
@@ -96,22 +99,41 @@ static int n_aliases;
 
 char *insmod_opt = NULL;
 char *depfile = NULL;
+char *pcimapfile = NULL;
 char *config_file = NULL;	/* Which file was actually used */
 time_t config_mtime;
 int root_check_off = 0;		/* Default, modules must be owned by root */
 static char *config_version;	/* Hack for config_add */
 int quick = 0;			/* Option -A */
 
+int flag_verbose;
+
+void verbose(const char *ctl,...)
+{
+	if (flag_verbose) {
+		va_list list;
+		va_start(list, ctl);
+		vprintf(ctl, list);
+		va_end(list); 
+		fflush(stdout);
+	}
+}
+
+
 /*
- *	Check to see if the existing modules.dep file needs updating,
+ *	Check to see if the existing modules.xxx files need updating,
  *	based on the timestamps of the modules and the config file.
  */
 static time_t dep_mtime;
-static int check_update (const char *file, const struct stat *sb, int flag)
+static time_t pcimap_mtime;
+static int check_update (const char *file, const struct stat *sb)
 {
 	int len = strlen(file);
 
-	if (sb->st_mtime <= dep_mtime)
+	if (!S_ISREG(sb->st_mode))
+		return 0;
+	if (sb->st_mtime <= dep_mtime &&
+	    sb->st_mtime <= pcimap_mtime)
 		return 0;
 
 	if (len > 2 && !strcmp(file + len - 2, ".o"))
@@ -127,7 +149,7 @@ static int check_update (const char *file, const struct stat *sb, int flag)
 
 static int need_update (const char *force_ver, const char *base_dir)
 {
-	struct stat statdep, tmp;
+	struct stat statdep, statpcimap, tmp;
 	char dep[PATH_MAX];
 	uname (&uts_info);
 	if (!force_ver)
@@ -139,7 +161,8 @@ static int need_update (const char *force_ver, const char *base_dir)
 		/* That's just silly. */
 		return 1;
 
-	if (stat (depfile, &statdep))
+	if (stat (depfile, &statdep) ||
+	    stat (pcimapfile, &statpcimap))
 		/* No dependency file yet, so we need to build it. */
 		return 1;
 
@@ -147,13 +170,15 @@ static int need_update (const char *force_ver, const char *base_dir)
 	    stat ("/etc/conf.modules", &tmp))
 		return 1;
 
-	if (tmp.st_mtime > statdep.st_mtime)
+	if (tmp.st_mtime > statdep.st_mtime ||
+	    tmp.st_mtime > statpcimap.st_mtime)
 		/* Config file is newer. */
 		return 1;
 
 	dep_mtime = statdep.st_mtime;
+	pcimap_mtime = statpcimap.st_mtime;
 	sprintf (dep, "%s/lib/modules/%s", base_dir, force_ver);
-	return ftw (dep, check_update, 2);
+	return xftw (dep, check_update);
 }
 
 
@@ -387,6 +412,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 	char *version;
 	char *type;
 	char depfile_tmp[PATH_MAX];
+	char pcimapfile_tmp[PATH_MAX];
 	char **glb;
 	char old_name[] = "/etc/conf.modules";
 	int conf_file_specified = 0;
@@ -452,6 +478,8 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 	 * [add] below module module1 ...
 	 *	Specify additional modules needed to be able to load a module
 	 *
+	 * [add] prune filename ...
+	 *
 	 * [add] probe name module1 ...
 	 *	When "name" is requested, modprobe tries to install each
 	 *	module in the list until it succeeds.
@@ -529,6 +557,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 	config_version = xstrdup(version);
 
 	depfile_tmp[0] = '\0';
+	pcimapfile_tmp[0] = '\0';
 
 	/* Only read the default entries on the first file */
 	if (depth == 0) {
@@ -558,6 +587,12 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 		if (all && below[0])
 			n_belowlist = build_list(below, &belowlist, version, 0);
 
+		/*
+		 * Build predef prune list
+		 */
+		if (prune[0])
+			n_prunelist = build_list(prune, &prunelist, version, 0);
+
 		/*
 		 * Build predef aliases
 		 */
@@ -648,6 +683,11 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			strncpy(depfile_tmp, envpath, sizeof(depfile_tmp));
 		}
 
+		/* PCIMAPPATH is intended for testing only */
+		if ((envpath = getenv("PCIMAPPATH")) != NULL) {
+			strncpy(pcimapfile_tmp, envpath, sizeof(pcimapfile_tmp));
+		}
+
 	}	/* End of depth == 0 */
 
 	if (conf_file ||
@@ -976,6 +1016,14 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			one_err = 0;
 		}
 
+		/*
+		 * prune
+		 */
+		else if (all && !assgn && strcmp(parm, "prune") == 0) {
+			decode_list(&n_prunelist, &prunelist, arg, adding, version, 0);
+			one_err = 0;
+		}
+
 		/*
 		 * probe
 		 */
@@ -1099,6 +1147,24 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			one_err = 0;
 		}
 
+		/*
+		 * pcimapfile=
+		 */
+		else if (assgn && (strcmp(parm, "pcimapfile") == 0) &&
+			 (pcimapfile_tmp[0] == '\0')) {
+			/*
+			 * The default value for the pcimapfile parameter is:
+			 *
+			 * pcimapfile=/lib/modules/`uname -r`/modules.pcimap
+			 *
+			 * If the config file exist but doesn't have a pcimapfile
+			 * specification, the default is used since modprobe
+			 * can't work without one.
+			 */
+			strcpy(pcimapfile_tmp, arg);
+			one_err = 0;
+		}
+
 		/*
 		 * keep
 		 */
@@ -1247,6 +1313,36 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 			depfile = g.pathv[0];
 	}
 
+	/*
+	 * Make sure we have a definition of "pcimapfile"
+	 */
+	if (pcimapfile_tmp[0] == '\0') {
+		/*
+		 * Specification: config file / no pcimapfile parameter
+		 * The default value for the pcimapfile parameter is:
+		 *
+		 * pcimapfile=/lib/modules/`uname -r`/modules.pcimap
+		 *
+		 * If the config file exists but lack a pcimapfile
+		 * specification, the default value is used since
+		 * the system can't work without one.
+		 */
+		sprintf(pcimapfile_tmp, "%s/lib/modules/%s/modules.pcimap",
+			(base_dir ? base_dir : ""), version);
+		pcimapfile = xstrdup(pcimapfile_tmp);
+	} else { /* pcimapfile defined in modules.conf */
+		/*
+		 * If we have a pcimapfile definition in the configuration file
+		 * we must resolve any shell meta-chars in its value.
+		 */
+		if (meta_expand(pcimapfile_tmp, &g, base_dir, version))
+			ret = -1;
+		else if (!g.pathv || g.pathv[0] == NULL)
+			ret = -1;
+		else
+			pcimapfile = g.pathv[0];
+	}
+
 	return ret;
 }
 
@@ -1283,7 +1379,7 @@ const char *looking_for;
 /*
  *	Add a file name if it exist
  */
-static int config_add(const char *file, const struct stat *sb, int flag)
+static int config_add(const char *file, const struct stat *sb)
 {
 	int i;
 	int npaths = 0;
@@ -1294,7 +1390,7 @@ static int config_add(const char *file, const struct stat *sb, int flag)
 		char **p;
 		char full[PATH_MAX];
 
-		if (flag != FTW_D)
+		if (!S_ISDIR(sb->st_mode))
 			return 0;
 		sprintf(full, "%s/%s", file, looking_for);
 
@@ -1306,10 +1402,7 @@ static int config_add(const char *file, const struct stat *sb, int flag)
 			paths[npaths++] = *p;
 		}
 	} else { /* normal path match or match with "*" */
-		if (flag != FTW_F)
-			return 0;
-
-		if (strstr(file, "modules.dep") != NULL)
+		if (!S_ISREG(sb->st_mode))
 			return 0;
 
 		if (strcmp(looking_for, "*") != 0) {
@@ -1331,7 +1424,7 @@ static int config_add(const char *file, const struct stat *sb, int flag)
 	for (i = 0; i < npaths; ++i) {
 		struct stat sbuf;
 
-		if (flag == FTW_D) {
+		if (S_ISDIR(sb->st_mode)) {
 			if (stat(paths[i], &sbuf) == 0)
 				sb = &sbuf;
 		}
@@ -1363,7 +1456,7 @@ static int config_add(const char *file, const struct stat *sb, int flag)
 				for (j = i + 1; j < npaths; ++j)
 					free(paths[j]);
 				free(paths);
-				return 1; /* finish ftw */
+				return 1; /* finish xftw */
 			}
 		}
 	    next:
@@ -1416,13 +1509,13 @@ GLOB_LIST *config_lstmod(const char *match, const char *type, int first_only)
 			sprintf(this, "%s/%s/%s", modpath[i].path,
 						  modpath[i].type, match);
 			if (stat(this, &sb) == 0 &&
-			    config_add(this, &sb, FTW_F))
+			    config_add(this, &sb))
 				break;
 			/* End fix for "2.1.121 syntax */
 
 			sprintf(this, "%s/%s", modpath[i].path, match);
 			if (stat(this, &sb) == 0 &&
-			    config_add(this, &sb, FTW_F))
+			    config_add(this, &sb))
 				break;
 		}
 
@@ -1430,28 +1523,15 @@ GLOB_LIST *config_lstmod(const char *match, const char *type, int first_only)
 #if 0
 		/* Fix for "2.1.121 syntax */
 		sprintf(this, "%s/%s", modpath[i].path, modpath[i].type);
-		if (stat(this, &sb) == 0 && S_ISDIR(sb.st_mode))
+		if (stat(this, &sb) == 0 && S_ISDIR(sb->st_mode))
 			path = this;
 		else
 		/* End fix for "2.1.121 syntax */
 #endif
 			path = modpath[i].path;
 
-		/*
-		 * Different definitions for the called function
-		 * on different versions of include files :-(
-		 * gcc -Wall might complain, but it _is_ OK!
-		 */
-		if ((ret = ftw(path, config_add, 32 /* whatever */))) {
-			/*
-			 * Some versions of ftw don't like non-existant paths
-			 */
-			if (errno == ENOENT)
-				ret = 0; /* Pretend it didn't happen */
-			else {
-				perror(path);
-				break;
-			}
+		if ((ret = xftw(path, config_add))) {
+			break;
 		}
 	}
 	if (ret >= 0) {
